declare module 'monaco-emacs/emacs/actions' {
	import * as monaco from 'monaco-editor';
	import { EmacsExtension } from 'monaco-emacs/emacs/index';
	export const SOURCE = "extension.emacs";
	export abstract class BaseAction {
	    description: string;
	    abstract run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class KillSelection extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension): void;
	}
	export class KillLines extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class InsertLineBelow extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class InsertLineAfter extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class SetMark extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension): void;
	} class MoveCursorAction extends BaseAction {
	    direction: string;
	    unit: string;
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class MoveCursorUp extends MoveCursorAction {
	    direction: string;
	}
	export class MoveCursorDown extends MoveCursorAction {
	    direction: string;
	}
	export class MoveCursorLeft extends MoveCursorAction {
	    direction: string;
	}
	export class MoveCursorRight extends MoveCursorAction {
	    direction: string;
	}
	export class MoveCursorToLineEnd extends MoveCursorAction {
	    direction: string;
	}
	export class MoveCursorToLineStart extends MoveCursorAction {
	    direction: string;
	}
	export class MoveCursorWordRight extends MoveCursorRight {
	    unit: string;
	}
	export class MoveCursorWordLeft extends MoveCursorLeft {
	    unit: string;
	}
	export class MoveCursorBottom extends MoveCursorAction {
	    direction: string;
	}
	export class MoveCursorTop extends MoveCursorAction {
	    direction: string;
	}
	export class KeyBoardQuit extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension): void;
	} class HistoryAction extends BaseAction {
	    action: string;
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class UndoAction extends HistoryAction {
	    action: string;
	}
	export class RedoAction extends HistoryAction {
	    action: string;
	}
	export class Yank extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class YankSelectionToRing extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class YankRotate extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class RevealEditorAction extends BaseAction {
	    revealFunction: string;
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class RevealToTopAction extends RevealEditorAction {
	    revealFunction: string;
	}
	export class RevealToCenterAction extends RevealEditorAction {
	    revealFunction: string;
	}
	export class RevealToBottomAction extends RevealEditorAction {
	    revealFunction: string;
	}
	export class InsertTabs extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class RotateCursorOnScreen extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class GotoLine extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension): void;
	}
	export class InvertSelection extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class Search extends BaseAction {
	    showReplace: boolean;
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export class SearchReplace extends Search {
	    showReplace: boolean;
	}
	export class DeleteLines extends BaseAction {
	    run(editor: monaco.editor.IStandaloneCodeEditor, ext: EmacsExtension, repeat: number): void;
	}
	export {};

}
declare module 'monaco-emacs/emacs/commands' {
	import * as monaco from 'monaco-editor';
	import { EmacsExtension } from 'monaco-emacs/emacs/';
	export type EditorCommand = {
	    action?: string | string[];
	    description?: string;
	    run?: (editor: monaco.editor.IStandaloneCodeEditor, extension: EmacsExtension, repeat: number) => void;
	};
	export interface ICommandMapping {
	    [key: string]: EditorCommand;
	}
	export const prefixPreservingKeys: {
	    "M-g": boolean;
	    "C-x": boolean;
	    "C-q": boolean;
	    "C-u": boolean;
	};
	export const COMMANDS: ICommandMapping;
	export function executeCommand(ext: EmacsExtension, command: EditorCommand, inputBuffer: string): void;
	export function registerGlobalCommand(key: string, command: EditorCommand): void;
	export function getAllMappings(): {
	    [key: string]: string;
	};

}
declare module 'monaco-emacs/emacs/util' {
	import * as monaco from 'monaco-editor';
	export const modifierKeys: {
	    [key: string]: string;
	};
	export function monacoToEmacsKey(ev: monaco.IKeyboardEvent): string;
	export interface Event<T> {
	    (listener: (e: T) => any, thisArgs?: any, disposables?: monaco.IDisposable[]): monaco.IDisposable;
	}
	export namespace Event {
	    const None: Event<any>;
	}
	export class Emitter<T> implements monaco.Emitter<T> {
	    private _event;
	    private _disposed;
	    private _listeners;
	    readonly event: Event<T>;
	    fire(ev?: T): any;
	    dispose(): void;
	}

}
declare module 'monaco-emacs/emacs/basicInput' {
	import * as monaco from 'monaco-editor';
	export class BasicInputWidget implements monaco.IDisposable {
	    private _dom;
	    private _input;
	    private _messageDom;
	    private _style;
	    private _pendingPromise;
	    constructor();
	    getDomNode(): HTMLElement;
	    getId(): string;
	    getPosition(): {
	        preference: monaco.editor.OverlayWidgetPositionPreference;
	    };
	    private addListeners;
	    private onKeyDown;
	    private onBlur;
	    private showWidgetAndFocus;
	    private cleanup;
	    getInput(message: string): Promise<string>;
	    dispose(): void;
	}

}
declare module 'monaco-emacs/emacs/state' {
	export const EAT_UP_KEY = "==-==";
	export class State {
	    private _inargumentMode;
	    private _prefixKey;
	    private _inputBuffer;
	    private _lastInputBuffer;
	    private _killRing;
	    updateAndGetKey(key: string): string;
	    private resetState;
	    private updateCuMode;
	    updateStateOnExecution(reset?: boolean): void;
	    getInputBuffer(): string;
	    addToRing(str: string): void;
	    growRingTop(str: string): void;
	    getFromRing(n?: number): string;
	    popFromRing(): string;
	    getReadableState(): string;
	}

}
declare module 'monaco-emacs/emacs/index' {
	import * as monaco from 'monaco-editor';
	import { State } from 'monaco-emacs/emacs/state';
	export class EmacsExtension implements monaco.IDisposable {
	    private _editor;
	    private _disposables;
	    private _inSelectionMode;
	    private _changeDisposable;
	    private _throttledScroll;
	    private _intialCursorType;
	    private _intialCursorBlinking;
	    private _basicInputWidget;
	    private _state;
	    private _onDidMarkChange;
	    onDidMarkChange: import("./util").Event<boolean>;
	    private _onDidChangeKey;
	    onDidChangeKey: import("./util").Event<string>;
	    constructor(editor: monaco.editor.IStandaloneCodeEditor);
	    start(): void;
	    readonly state: State;
	    getEditor(): monaco.editor.IStandaloneCodeEditor;
	    private addListeners;
	    private cancelKey;
	    private onKeyDown;
	    onEditorScroll(): void;
	    private onContentChange;
	    clearState(): void;
	    selectionMode: boolean;
	    getCursorAnchor(): monaco.Position;
	    getBasicInput(message: string): Promise<string>;
	    dispose(): void;
	}

}
declare module 'monaco-emacs/index' {
	import { EmacsExtension } from 'monaco-emacs/emacs';
	export { registerGlobalCommand, getAllMappings } from 'monaco-emacs/emacs/commands';
	import * as Actions from 'monaco-emacs/emacs/actions';
	export { EmacsExtension, Actions };
	export default EmacsExtension;

}
